#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Calculates Yoga Vimsottari
"""
from collections import OrderedDict as Dict
from jhora import const,utils
from jhora.panchanga import drik
sidereal_year = const.sidereal_year #const.savana_year #  # some say 360 days, others 365.25 or 365.2563 etc
vimsottari_dict = { 8:[(3,12,21), 7], 5: [(4,13,22),20], 0:[(5,14,23), 6], 1:[(6,15,24), 10], 2:[(7,16,25), 7], 
                   7:[(8,17,26), 18], 4:[(9,18,27), 16], 6:[(1,10,19), 19], 3:[(2,11,20), 17] }
human_life_span_for_vimsottari_dhasa = const.human_life_span_for_vimsottari_dhasa
### --- Vimoshatari functions
def vimsottari_adhipathi(yoga_index):
    for key,(yoga_list,durn) in vimsottari_dict.items():
        if yoga_index in yoga_list:
            return key,durn 
def vimsottari_next_adhipati(lord,dirn=1):
    """Returns next guy after `lord` in the adhipati_list"""
    current = const.vimsottari_adhipati_list.index(lord)
    next_index = (current + dirn) % len(const.vimsottari_adhipati_list)
    return const.vimsottari_adhipati_list[next_index]

def vimsottari_dasha_start_date(jd,place):
    """Returns the start date of the mahadasa which occured on or before `jd`"""
    _,_,_,birth_time_hrs = utils.jd_to_gregorian(jd)
    _yoga = drik.yogam(jd, place)
    y_frac = utils.get_fraction(_yoga[1], _yoga[2], birth_time_hrs)
    #print('yoga',_yoga,'birth_time_hrs',birth_time_hrs,'yoga_fracion',y_frac)
    lord,res = vimsottari_adhipathi(_yoga[0])          # ruler of current nakshatra
    period_elapsed = (1-y_frac)*res*sidereal_year
    start_jd = jd - period_elapsed      # so many days before current day
    #print('lord,res,period_elapsed,start_date',lord,res,period_elapsed,utils.jd_to_gregorian(start_date))
    return [lord, start_jd]

def vimsottari_mahadasa(jdut1,place):
    """List all mahadashas and their start dates"""
    lord, start_date = vimsottari_dasha_start_date(jdut1,place)
    retval = Dict()
    for i in range(9):
        retval[lord] = start_date; lord_duration = vimsottari_dict[lord][1]
        start_date += lord_duration * sidereal_year
        lord = vimsottari_next_adhipati(lord)
    return retval

def _vimsottari_bhukti(maha_lord, start_date,antardhasa_option=1):
    """Compute all bhuktis of given nakshatra-lord of Mahadasa
    and its start date"""
    lord = maha_lord
    if antardhasa_option in [3,4]:
        lord = vimsottari_next_adhipati(lord, dirn=1) 
    elif antardhasa_option in [5,6]:
        lord = vimsottari_next_adhipati(lord, dirn=-1) 
    dirn = 1 if antardhasa_option in [1,3,5] else -1
    dhasa_lord_duration = vimsottari_dict[maha_lord][1]
    retval = Dict()
    for _ in range(len(vimsottari_dict)):
        retval[lord] = start_date; bhukthi_duration = vimsottari_dict[lord][1]
        factor = bhukthi_duration * dhasa_lord_duration / human_life_span_for_vimsottari_dhasa
        start_date += factor * sidereal_year
        lord = vimsottari_next_adhipati(lord,dirn)

    return retval

# North Indian tradition: dasa-antardasa-pratyantardasa
# South Indian tradition: dasa-bhukti-antara-sukshma
def _vimsottari_antara(maha_lord, bhukti_lord, start_date):
    """Compute all antaradasas from given bhukit's start date.
    The bhukti's lord and its lord (mahadasa lord) must be given"""
    lord = bhukti_lord
    retval = Dict()
    for _ in range(9):
        retval[lord] = start_date
        factor = vimsottari_dict[lord] * (vimsottari_dict[maha_lord] / human_life_span_for_vimsottari_dhasa)
        factor *= (vimsottari_dict[bhukti_lord] / human_life_span_for_vimsottari_dhasa)
        start_date += factor * sidereal_year
        lord = vimsottari_next_adhipati(lord)

    return retval


def _where_occurs(jd, some_dict):
    """Returns minimum key such that some_dict[key] < jd"""
    # It is assumed that the dict is sorted in ascending order
    # i.e. some_dict[i] < some_dict[j]  where i < j
    for key in reversed(some_dict.keys()):
        if some_dict[key] < jd: return key


def compute_vimsottari_antara_from(jd, mahadashas):
    """Returns antaradasha within which given `jd` falls"""
    # Find mahadasa where this JD falls
    i = _where_occurs(jd, mahadashas)
    # Compute all bhuktis of that mahadasa
    bhuktis = _vimsottari_bhukti(i, mahadashas[i])
    # Find bhukti where this JD falls
    j = _where_occurs(jd, bhuktis)
    # JD falls in i-th dasa / j-th bhukti
    # Compute all antaras of that bhukti
    antara = _vimsottari_antara(i, j, bhuktis[j])
    return (i, j, antara)

def get_dhasa_bhukthi(jd,place,use_tribhagi_variation=False,antardhasa_option=1):
    """
        provides Yoga Vimsottari dhasa bhukthi for a given date in julian day (includes birth time)
        This is vimsottari but based on yogam instead of nakshathra
        @param jd: Julian day for birthdate and birth time
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param use_tribhagi_variation: False (default), True means dhasa bhukthi duration in three phases 
        @param antardhasa_option:
            1 => dhasa lord - forward
            2 => dhasa lord - backward
            3 => next dhasa lord - forward (Default)
            4 => next dhasa lord - backward
            5 => prev dhasa lord - forward
            6 => prev dhasa lord - backward
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    global human_life_span_for_vimsottari_dhasa
    # jd is julian date with birth time included
    _dhasa_cycles = 1
    _tribhagi_factor = 1
    if use_tribhagi_variation:
        _tribhagi_factor = 1./3.
        _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
        human_life_span_for_vimsottari_dhasa *= _tribhagi_factor
        for k,v in vimsottari_dict.items():
            vimsottari_dict[k] = round(v*_tribhagi_factor,2)
    _,_,_,tz = place
    jdut1 = jd - tz/24
    dashas = vimsottari_mahadasa(jdut1,place)
    dl = list(dashas.values()); de = dl[1]
    y,m,h,_ = utils.jd_to_gregorian(jd); p_date1 = drik.Date(y,m,h)
    y,m,h,_ = utils.jd_to_gregorian(de); p_date2 = drik.Date(y,m,h)
    vim_bal = utils.panchanga_date_diff(p_date1, p_date2)
    dhasa_bukthi=[]
    for _ in range(_dhasa_cycles):
        for i in dashas:
            bhuktis = _vimsottari_bhukti(i, dashas[i],antardhasa_option=antardhasa_option)
            dhasa_lord = i
            for j in bhuktis:
                bhukthi_lord = j
                jd1 = bhuktis[j]+tz/24
                y, m, d, h = utils.jd_to_gregorian(jd1)#swe.revjul(round(jd1 + tz/24))
                """ TODO: Need to figure out passing date and time string to UI, main.py and pvr_tests.py """
                date_str = '%04d-%02d-%02d' %(y,m,d)+' '+utils.to_dms(h,as_string=True)
                bhukthi_start = date_str
                dhasa_bukthi.append([dhasa_lord,bhukthi_lord,bhukthi_start]) 
    return vim_bal,dhasa_bukthi

'------ main -----------'
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.vimsottari_tests()




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import itertools
from jhora import const,utils
from jhora.horoscope.chart import house
from jhora.horoscope.transit import tajaka
""" Tajaka Yogas """
def ishkavala_yoga(planet_to_house_dict):
    """
         Ishkavala Yoga
            If planets occupy only kendras (1st, 4th, 7th and 10th houses) and panapharas (2nd, 5th,
            8th and 11th houses) and if apoklimas (3rd, 6th, 9th and 12th houses) are empty, then
            this yoga is present. This yoga gives wealth, happiness and good fortune.
        @param planet_to_house_dict: Example {0:1.1:2,2:0,..'L':1}
        @param asc_house: Raasi index of the ascendant/Lagnam
        @return: True/False - whether ishkaval yoga is present or not  
    """
    """ TODO Not Working """
    asc_house = planet_to_house_dict[const._ascendant_symbol]
    slq = list(set([(asc_house+h-1)%12 for h in [1,2,4,5,7,8,10,11]]))
    sph = list(set([house.get_relative_house_of_planet(asc_house,h)-1 for h in list(planet_to_house_dict.values())[:-1]]))
    yoga_present = all(el in slq for el in sph)
    return yoga_present
def induvara_yoga(planet_to_house_dict):
    """
        Induvara Yoga
            If planets occupy only apoklimas (3rd, 6th, 9th and 12th houses) and if kendras (1st, 4th,
            7th and 10th houses) and panapharas (2nd, 5th, 8th and 11th houses) are empty, then this
            yoga is present. This yoga gives disappointments, worries and illnesses.    
        @param planet_to_house_dict: Example {0:1.1:2,2:0,..'L':1}
        @param asc_house: Raasi index of the ascendant/Lagnam
        @return: True/False - whether induvara yoga is present or not  
    """
    asc_house = planet_to_house_dict[const._ascendant_symbol]
    slq = list(set([(asc_house+h-1)%12 for h in [3,6,9,12]]))
    sph = list(set([house.get_relative_house_of_planet(asc_house,h)-1 for h in list(planet_to_house_dict.values())[:-1]]))
    yoga_present = all(el in slq for el in sph)
    #print('sph',sph,'slq',slq)
    return yoga_present
def ithasala_yoga(planet_positions,planet1,planet2):
    """
        Ithasala Yoga
            If two planets have an aspect and if the faster moving planet83 is less advanced in its
            rasi than the slower moving planet, then we have an ithasala yoga between the two.        
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @param asc_house: Raasi index of the ascendant/Lagnam
        @return: True/False - whether yoga is present or not  
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    #print(house_planet_dict)
    chk1 = tajaka.planets_have_aspects(house_planet_dict, planet1, planet2)
    chk2,ithasala_type = tajaka.both_planets_within_their_deeptamsa(planet_positions,planet1, planet2)
    chk3 = tajaka.both_planets_approaching(planet_positions,planet1,planet2)
    yoga_present = chk1 and chk2 and chk3
    return yoga_present, ithasala_type
def eesarpha_yoga(planet_positions,planet1,planet2):
    """
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @param asc_house: Raasi index of the ascendant/Lagnam
        @return: True/False - whether yoga is present or not  
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    #print(house_planet_dict)
    chk1 = tajaka.planets_have_aspects(house_planet_dict, planet1, planet2)
    chk2,ithasala_type = tajaka.both_planets_within_their_deeptamsa(planet_positions,planet1, planet2)
    chk3 = tajaka.both_planets_approaching(planet_positions,planet1,planet2)
    yoga_present = chk1 and chk2 and (not chk3)
    return yoga_present
def _check_nakta_yoga_1(planet_positions,planet1,planet2):
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    excluded_planets = [const._ascendant_symbol,'7','8']
    yp = str(planet1) not in excluded_planets and str(planet2) not in excluded_planets
    yp = yp and not ithasala_yoga(planet_positions,int(planet1),int(planet2))[0]
    yp = yp and not eesarpha_yoga(planet_positions, int(planet1), int(planet2))
    yp = yp and int(planet1) not in tajaka.aspects_of_the_planet(house_planet_dict,int(planet2))
    return yp
def _check_nakta_yoga(planet_positions, planet, p1, p2):
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    yp1 = _check_nakta_yoga_1(planet_positions, p1, p2)
    if not yp1:
        return False
    yp2 = ithasala_yoga(planet_positions, int(planet), int(p1))[0]
    yp3 = ithasala_yoga(planet_positions, int(planet), int(p2))[0]
    yp = yp1 and yp2 and yp3
    #print('_check_nakta_yoga_1(planet_positions, p1, p2)',planet,p1,p2,yp1,yp2,yp3,yp)
    """
    if not yp:
        return False
    yp = yp and str(p2) in tajaka.benefic_aspects_of_the_planet(house_planet_dict, planet)
    if not yp:
        return False
    yp = yp and const.order_of_planets_by_speed.index(planet) > const.order_of_planets_by_speed.index(int(p1)) and \
            const.order_of_planets_by_speed.index(planet) > const.order_of_planets_by_speed.index(int(p2))
    if yp:
        p_long = planet_positions[planet+1][1][1]
        p1_long = planet_positions[int(p1)+1][1][1]
        p2_long = planet_positions[int(p2)+1][1][1]
        yp = yp and p_long < p1_long and p_long < p2_long
    """
    return yp
def _get_nakta_triples(ithasala_pairs):
    from collections import defaultdict
    iy_list = defaultdict(list)
    [iy_list[k].append(v) for k,v,_ in ithasala_pairs]
    iy_list = [(k,list(itertools.combinations(lst,2))) for k,lst in iy_list.items() if len(lst)>1]
    print('_get_nakta_triples iy_list',iy_list)
    iy_list1 = defaultdict(list)
    [iy_list1[v].append(k) for k,v,_ in ithasala_pairs]
    iy_list1 = [(k,list(itertools.combinations(lst,2))) for k,lst in iy_list1.items() if len(lst)>1]
    print('_get_nakta_triples iylist1',iy_list1)
    iy_list += iy_list1
    print('_get_nakta_triples iy_list',iy_list)
    _nakta_triples = iy_list # utils.flatten_list(iy_list)
    return _nakta_triples
def get_nakta_yoga_planet_triples(planet_positions):
    """
        nakta yoga between p2 and p3 if 
             p1 & p2 and p1 & p3 have ithasala yoga between them 
             but p2 and p3 have no aspects
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of natka yoga triples
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    #print('ithasala_yoga_planet_pairs',iy_pairs)
    _nakta_triples = _get_nakta_triples(iy_pairs)
    #print(_nakta_triples)
    nt = []
    for planet,p_list in _nakta_triples:
        p_long = planet_positions[planet+1][1][1]
        for p1,p2 in p_list:
            p1_long = planet_positions[p1+1][1][1]
            p2_long = planet_positions[p2+1][1][1]
            #print('checking triples',planet,p_long,p1,p1_long,p2,p2,p_long)
            if not tajaka.planets_have_aspects(house_planet_dict, p1, p2) and \
                    p_long < p1_long and p_long < p2_long:
                nt.append([planet,(p1,p2)])
                #print('found natka triple',planet,p1,p2)
    _nakta_triples = utils.flatten_list(nt)
    #print('natka planet triples',_nakta_triples)
    return _nakta_triples
def check_yamaya_yoga(planet,planet1,planet2,planet_positions):
    planet_house_dict = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    #print(planet_house_dict)
    house_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    #print(house_planet_list)
    # Step-1: check planet1 and planet2 have no aspects
    chk1 = not tajaka.planets_have_aspects(house_planet_list, planet1, planet2)
    if not chk1:
        return False
    #Step-2: Check planet has aspect with planet1 and planet 2
    chk2_1 = tajaka.planets_have_aspects(house_planet_list,planet,planet1)
    chk2_2 = tajaka.planets_have_aspects(house_planet_list,planet,planet2)
    chk2 = chk2_1 and chk2_2 
    if not chk2:
        return False
    #Step-3: Planet forms ithasala pair with planet1 and planet2
    chk3_1 = ithasala_yoga(planet_positions,planet,planet1)
    chk3_2 = ithasala_yoga(planet_positions,planet,planet2)
    chk3 = chk3_1 and chk3_2
    if not chk3:
        return False
    # Step-4: check if planet has higher longitude than planet1/planet2
    p_long = planet_positions[planet+1][1][1]
    p1_long = planet_positions[planet1+1][1][1]
    p2_long = planet_positions[planet2+1][1][1]
    chk4 = p_long > p1_long and p_long > p2_long
    #print(p_long,p1_long,p2_long,chk3)
    return chk4
def get_yamaya_yoga_planet_triples(planet_positions):
    """
        yamaya yoga between p2 and p3 if 
             p1 & p2 and p1 & p3 have ithasala yoga between them 
             but p2 and p3 have no aspects
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of yamaya yoga triples
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    print('ithasala_yoga_planet_pairs',iy_pairs)
    _yamaya_triples = _get_nakta_triples(iy_pairs)
    print('_yamaya_triples',_yamaya_triples)
    nt = []
    for planet,p_list in _yamaya_triples:
        p_long = planet_positions[planet+1][1][1]
        for p1,p2 in p_list:
            p1_long = planet_positions[p1+1][1][1]
            p2_long = planet_positions[p2+1][1][1]
            print('checking triples',planet,p_long,p1,p1_long,p2,p2,p_long)
            if not tajaka.planets_have_aspects(house_planet_dict, p1, p2) and \
                    p_long > p1_long and p_long > p2_long:
                nt.append([planet,(p1,p2)])
                #print('found yamaya triple',planet,p1,p2)
    _yamaya_triples = utils.flatten_list(nt)
    print('yamaya planet triples',_yamaya_triples)
    return _yamaya_triples
def get_eesarpha_yoga_planet_pairs(planet_positions):
    """
        Get eeasrpha yoga planet pairs
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of eesarpha yoga pairs        
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    com1=[]
    for p1,p2 in list(itertools.combinations([*range(7)],2)):
        iy = eesarpha_yoga(planet_positions, p1, p2)
        if iy:
           com1.append((p1,p2))
    return com1 
def get_ithasala_yoga_planet_pairs(planet_positions):
    """
        Get ithasala yoga planet pairs
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of ithasala yoga pairs        
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print('house_planet_dict',house_planet_dict)
    com1=[]
    for p1,p2 in list(itertools.combinations([*range(7)],2)):
        iy,iyt = ithasala_yoga(planet_positions, p1, p2)
        if iy:
           com1.append((p1,p2,iyt))
    return com1 
def get_manahoo_yoga_planet_pairs(planet_positions):
    """
        Get manahoo yoga planet pairs
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of manahoo yoga pairs        
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    my = []
    mars_or_saturn_houses = [planet_positions[3+1][1][0],planet_positions[6+1][1][0]]
    for p1,p2,_ in iy_pairs: # ithasala tuple is p1,p2,othasal_type
        p1_long = planet_positions[p1+1][1][1]
        p2_long = planet_positions[p2+1][1][1]
        faster_planet = p1
        faster_planet_long = p1_long
        if p2_long < p1_long:
            faster_planet = p2
            faster_planet_long = p2_long
        faster_planet_house = planet_positions[faster_planet+1][1][0]
        faster_planet_deeptaamsa_start,faster_planet_deeptaamsa_end = utils.deeptaamsa_range_of_planet(faster_planet, faster_planet_long)
        if faster_planet_house in mars_or_saturn_houses:
            m_s_index = mars_or_saturn_houses.index(faster_planet_house)
            m_s_long = planet_positions[m_s_index+1][1][1]
            if m_s_long > faster_planet_deeptaamsa_start and m_s_long < faster_planet_deeptaamsa_end:
                my.append([p1,p2,mars_or_saturn_houses[m_s_index]])
    return my
def get_kamboola_yoga_planet_pairs(planet_positions):
    """
        Get kamboola yoga planet pairs
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of kamboola yoga pairs        
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    iy_pairs = [(x,y) for x,y,_ in iy_pairs]
    #print(iy_pairs)
    ky_pairs = [ (x,y) for x,y in iy_pairs if x==1 or x==1]
    ky_planets = [ y for x,y in iy_pairs if x==1]+[ x for x,y in iy_pairs if y==1]    
    #print(ky_pairs, ky_planets)
    iy_ky_pairs = [(x,y) for kyp in ky_planets for x,y in iy_pairs if kyp==x or kyp==y]
    ky_check =any([kyp == x or kyp ==y for kyp in ky_planets for x,y in iy_pairs])
    return (ky_check,ky_pairs,iy_ky_pairs)
def get_gairi_kamboola_yoga_planet_pairs(planet_positions):
    """ TODO: to be implemented """
    return None
def get_khallasara_yoga_planet_pairs(planet_positions):
    """ TODO: to be implemented """
    return None
def get_radda_yoga_planet_pairs(planet_positions):
    """
        Get radda yoga planet pairs
        @param planet_positions: [ ['L',(7,12,3456)], [0,(4,112,3456)],...]]
        @return: List of radda yoga pairs        
    """
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    iy_pairs = [(x,y) for x,y,_ in iy_pairs]
    chk = []
    chk.append([x in charts.planets_in_combustion(planet_positions) or y in charts.planets_in_combustion(planet_positions) for x,y in iy_pairs])
    chk.append([x in charts.planets_in_retrograde(planet_positions) or y in charts.planets_in_retrograde(planet_positions) for x,y in iy_pairs])
    chk.append([const.house_strengths_of_planets[x][p_to_h[x]] < const._NEUTRAL_SAMAM or const.house_strengths_of_planets[y][p_to_h[y]]<2 for x,y in iy_pairs])
    import numpy as np
    ry_check= list(np.any(chk,axis=0))
    ry_pairs = [(x,y) for i,(x,y) in enumerate(iy_pairs) if ry_check[i]]
    return ry_pairs
def get_duhphali_kutta_yoga_planet_pairs(jd,place):
    """
        Get duhphali kutta yoga planet pairs
        @param jd: Julian Day Number
        @param place: drik.Place struct ('place name',latitude, longitude, timezone) 
        @return: List of duhphali kutta yoga pairs        
    """
    planet_positions = charts.divisional_chart(jd, place)
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print('house planet chart',house_planet_dict)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    iy_pairs = get_ithasala_yoga_planet_pairs(planet_positions)
    iy_pairs = [(x,y) for x,y,_ in iy_pairs]
    dky_pairs = []
    for x,y in iy_pairs:
        faster_planet = x
        slower_planet = y
        if const.order_of_planets_by_speed.index(x) < const.order_of_planets_by_speed.index(y):
            faster_planet = y
            slower_planet = x
        print('ithasala pair',faster_planet,slower_planet)
        chk = const.house_strengths_of_planets[faster_planet][p_to_h[faster_planet]] > const._FRIEND
        print('chk1',chk,const.house_strengths_of_planets[faster_planet][p_to_h[faster_planet]])
        if not chk:
            continue
        from jhora.horoscope.chart import strength
        pvb = strength.pancha_vargeeya_bala(jd, place)
        print('pvb',pvb[faster_planet],pvb[slower_planet])
        chk = chk & (pvb[faster_planet]  > const.pancha_vargeeya_bala_strength_threshold)
        print('chk2',chk)
        if not chk:
            continue
        chk = chk & (const.house_strengths_of_planets[slower_planet][p_to_h[slower_planet]] < const._EXALTED_UCCHAM)
        print('chk3',chk)
        if not chk:
            continue
        chk = chk & (pvb[slower_planet]  < const.pancha_vargeeya_bala_strength_threshold)
        print('chk3',chk)
        if not chk:
            continue
        else:
            dky_pairs.append((x,y))
    print('duhphali_kutta_yoga_planet_pairs',dky_pairs)
    return dky_pairs
def nakta_yoga(planet_positions,planet):
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(planet_positions)
    #print(house_planet_dict)
    ah,ap = tajaka.aspects_of_the_planet(house_planet_dict, planet)
    com = list(itertools.combinations(ap,2))
    com1 = []
    for p1,p2 in com:
        if _check_nakta_yoga(planet_positions, planet, p1, p2):
            com1.append((int(p1),int(p2)))
    com = com1[:]
    #print(planet,'combinations',com)
    yoga_present = len(com)>0
    return yoga_present,com
"""
    Ithasala yoga
        If two planets have an aspect and if the faster moving planet83 is less advanced in its
        rasi than the slower moving planet, then we have an ithasala yoga between the two.
"""
if __name__ == "__main__":
    pp = [['L',(1,5)],[0,(0,25)],[1,(0,20)],[2,(7,15)],[3,(0,23)],[4,(3,16)],[5,(2,13)],[6,(0,0)],[7,(0,0)],[8,(0,0)]]
    planet1 = 2
    planet2 = 5
    planet3 = 4
    planet_house_dict = utils.get_planet_house_dictionary_from_planet_positions(pp)
    print(check_yamaya_yoga(planet3, planet1, planet2, pp))
    exit()
    from jhora.panchanga import drik
    from jhora.horoscope.chart import charts
    jd_at_years = utils.julian_day_number((1993,6,1),(13,30,4))
    place = drik.Place('unknown',16+15.0/60,81+12.0/60,5.5)
    chart_67_rasi = charts.divisional_chart(jd_at_years, place, divisional_chart_factor=1)
    com = get_ithasala_yoga_planet_pairs(chart_67_rasi)
    print('ithasala combinations\n',com)
    com = get_eesarpha_yoga_planet_pairs(chart_67_rasi)
    print('eesarpha_yoga combinations\n',com)
    exit()
    chart_66 = ['0','','3','7','','4','8','','6','5/L','','2/1']
    chart_66 = ['3','4','','5','7','','8','6/2','','L/0','1','']
    print(chart_66)
    p_to_h = utils.get_planet_to_house_dict_from_chart(chart_66)
    print('p_to_h',p_to_h)
    asc_house = p_to_h[const._ascendant_symbol]
    print('asc_house',asc_house)
    print(ishkavala_yoga(p_to_h,asc_house))
    exit()
    from jhora.panchanga import drik
    from jhora.horoscope.chart import charts
    jd_at_dob = utils.julian_day_number((1972,6,1),(4,16,0))
    years = 21
    place = drik.Place('unknown',16+15.0/60,81+12.0/60,5.5)
    divisional_chart_factor = 1
    ayanamsa_mode = const._DEFAULT_AYANAMSA_MODE
    jd_at_years = utils.julian_day_number((1993,6,1),(13,30,4))
    dky = get_duhphali_kutta_yoga_planet_pairs(jd_at_years,place)
    exit()
    chart_67_rasi = charts.divisional_chart(jd_at_years, place, ayanamsa_mode, divisional_chart_factor=1)
    print(chart_67_rasi)
    house_planet_dict = utils.get_house_planet_list_from_planet_positions(chart_67_rasi)
    print(house_planet_dict)
    ry = get_radda_yoga_planet_pairs(chart_67_rasi)
    print(ry)
    exit()
    ky = get_kamboola_yoga_planet_pairs(chart_67_rasi)
    print('get_kamboola_yoga_planet_pairs',ky)
    exit()
    get_nakta_yoga_planet_triples(chart_67_rasi)
    get_yamaya_yoga_planet_triples(chart_67_rasi)
    com = get_manahoo_yoga_planet_pairs(chart_67_rasi)
    print('manahoo combinations\n',com)
    exit()
    com = get_eesarpha_yoga_planet_pairs(chart_67_rasi)
    print('eesarpha combinations\n',com)
    exit()
    com = []
    for planet in range(7):
        ty = nakta_yoga(chart_67_rasi,planet)
        if ty[0] and len(ty[1])>0:
            com.append([planet,ty[1]])
        print('nakta yoga',ty)
    print('nakta yoga',com)
    exit()
    chart_66 = ['0','','3','7','','4','8','','6','5/L','','2/1']
    print(chart_66)
    p_to_h = utils.get_planet_to_house_dict_from_chart(chart_66)
    print('p_to_h',p_to_h)
    asc_house = p_to_h[const._ascendant_symbol]
    print('asc_house',asc_house)
    print(ishkavala_yoga(p_to_h,asc_house))
    chart_66 = ['','','3/0/5','','','4/7/8','','','6','L','','2/1']
    print(chart_66)
    p_to_h = utils.get_planet_to_house_dict_from_chart(chart_66)
    print('p_to_h',p_to_h)
    asc_house = p_to_h[const._ascendant_symbol]
    print('asc_house',asc_house)
    print(induvara_yoga(p_to_h,asc_house))


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora.horoscope.chart import house, charts
from jhora import utils, const
from jhora.panchanga import drik
"""
    DO NOT USE THIS YET. UNDER TESTING...
    Based on Book: Hindu Predictive Astrology - by BV Raman
"""
def _baladrishta_checks(jd,place,divisional_chart_factor=1):
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print(h_to_p)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    print(p_to_h)
    asc_house = p_to_h['L']
    _relative_house = lambda sign: house.get_relative_house_of_planet(asc_house,sign)
    bd_checks = []
    """ malefics in last navamsa """
    """ Moon in kendra with malefics """
    bc = any([p_to_h[1] in q for q in [house.quadrants_of_the_raasi(p_to_h[m]) for m in const.natural_malefics]])
    bd_checks.append(bc)
    """ Moon in 7/8/12th house with malefics and without benefic aspects"""
    bc = (_relative_house(p_to_h[1]) in [7,8,12]) and \
            any([p_to_h[1]==p_to_h[m] for m in const.natural_malefics]) and \
            any([1 in q for q in [house.aspected_planets_of_the_planet(h_to_p, b) for b in const.natural_benefics]])
    bd_checks.append(bc)
    """ Malefics in 2,6,8,12 house """
    bc = all([mh in [2,6,8,12] for mh in [_relative_house(p_to_h[m]) for m in charts.malefics(jd, place)]])
    bd_checks.append(bc)
    """ Weak moon in ascendant or 8th and Malefics in quadrants """
    bc = const.house_strengths_of_planets[1][p_to_h[1]] < const._FRIEND and \
                ( (p_to_h[1] == asc_house) or (_relative_house(p_to_h[1])==8) ) and \
                any([m in house.quadrants_of_the_raasi(p_to_h[1]) for m in const.natural_malefics])
    
    bd_checks.append(bc)
    """ Moon in ascendant, Mars in 8th, Sun in 9th, and Saturn in 12th """
    bc = (asc_house==p_to_h[1]) and (_relative_house(p_to_h[0])==9) and \
         (_relative_house(p_to_h[2])==8) and \
         (_relative_house(p_to_h[6])==12)
    bd_checks.append(bc)
    """ Moon in 8th, Mars in 7th, Rahu in 9th, and Jupiter in 3rd """
    bc = (_relative_house(p_to_h[1])==8) and \
         (_relative_house(p_to_h[2])==7) and \
         (_relative_house(p_to_h[7])==9) and \
         (_relative_house(p_to_h[4])==3)
    bd_checks.append(bc)
    return bd_checks
def _alpayu_checks(jd,place,divisional_chart_factor=1):
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print(h_to_p)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    print(p_to_h)
    asc_house = p_to_h['L']
    _relative_house = lambda sign: house.get_relative_house_of_planet(asc_house,sign)
    _house_owner = lambda sign:house.house_owner_from_planet_positions(planet_positions,sign)
    """ Saturn in 8th, Mars in 5th and Kethu with Ascendant/1st """
    ad_checks = []
    ac = (_relative_house(p_to_h[6])==8) and \
         (_relative_house(p_to_h[2])==5) and \
         (asc_house==p_to_h[8])
    ad_checks.append(ac)
    ac = all([mh in [6,8,12] for mh in [_relative_house(p_to_h[m]) for m in charts.malefics(jd, place)]])
    ad_checks.append(ac)
    """ Saturn in Lagna and the Sun, Moon and Mars in 7th """
    ac = (p_to_h[1]==p_to_h[2]) and (p_to_h[0]==p_to_h[1])\
         (_relative_house(p_to_h[2])==7) and \
         (asc_house==p_to_h[6])
    ad_checks.append(ac)
    """ Lord of 8th in Ascendant with Kethu"""
    eighth_lord = _house_owner((asc_house+7)%12)
    ac = asc_house==p_to_h[eighth_lord] and asc_house==p_to_h[8]
    ad_checks.append(ac)
    """ Malefics with Ascendant and moon is in conjunction """
    """ TODO: Any malefic or All Malefics??? """
    ac = any([p_to_h[1] == p_to_h[m] and asc_house == p_to_h[m] for m in const.natural_malefics])
    ad_checks.append(ac)
    """ Lord of 8th in Ascendant and Lagna Lord is powerless """
    eighth_lord = _house_owner((asc_house+7)%12)
    lagna_lord = _house_owner(asc_house)
    #print(eighth_lord,lagna_lord,const.house_strengths_of_planets[lagna_lord][p_to_h[lagna_lord]])
    ac = (p_to_h[eighth_lord]==asc_house) and (const.house_strengths_of_planets[lagna_lord][p_to_h[lagna_lord]] < const._FRIEND)
    ad_checks.append(ac)
    """ Lagna in Scorpio/Vrichigam with Sun and Jupiter, 8th lord in quardrants """
    ac = (asc_house==p_to_h[0]) and (asc_house==p_to_h[4] and (asc_house==7))
    eighth_lord = _house_owner((asc_house+7)%12)
    ac = ac and any([p_to_h[eighth_lord] in house.quadrants_of_the_raasi(asc_house)])
    #print(eighth_lord,p_to_h[eighth_lord],house.quadrants_of_the_raasi(asc_house))
    ad_checks.append(ac)
    """ Rahu/7th and Moon/8th and Jupiter/Ascendant""" 
    ac = _relative_house(p_to_h[7])==7 and \
         _relative_house(p_to_h[1])==8 and \
         (asc_house == p_to_h[4])
    ad_checks.append(ac)
    """ Saturn/Ascendant owned by malefic and benefics in [3,6,9,12] """
    ac = asc_house==p_to_h[6] and \
         any([m == lagna_lord for m in charts.malefics(jd, place)]) and \
         all([_relative_house(p_to_h[b]) in [3,6,9,12]  for b in charts.benefics(jd, place)])    
    ad_checks.append(ac)
    """ Lords of Asc and 8th conjoin in 8th with malefics and without beneficial aspect """
    ac = p_to_h[lagna_lord]==p_to_h[eighth_lord] and \
         p_to_h[lagna_lord]==(asc_house+7)%12 and \
         any([p_to_h[m] == p_to_h[lagna_lord] for m in charts.malefics(jd, place)]) and \
         any([lagna_lord in q for q in [house.aspected_planets_of_the_planet(h_to_p, b) for b in const.natural_benefics]])
    ad_checks.append(ac)
    return ad_checks
def _madhyayu_checks(jd,place,divisional_chart_factor=1):
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print(h_to_p)
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    print(p_to_h)
    asc_house = p_to_h['L']
    _relative_house = lambda sign: house.get_relative_house_of_planet(asc_house,sign)
    _house_owner = lambda sign:house.house_owner_from_planet_positions(planet_positions,sign)
    """ Lord of 3rd or 6th in kendra/quadrant """
    md_checks = []
    mc = p_to_h[_house_owner((asc_house+2)%12)] in house.kendra_aspects_of_the_raasi(asc_house) or \
         p_to_h[_house_owner((asc_house+5)%12)] in house.kendra_aspects_of_the_raasi(asc_house)
    md_checks.append(mc)
    """ Two of: lord of 8th, 10th 11th - are powerful """
    eighth_lord = _house_owner((asc_house+7)%12)
    tenth_lord = _house_owner((asc_house+9)%12)
    eleventh_lord = _house_owner((asc_house+10)%12)
    mc = sum([const.house_strengths_of_planets[p][p_to_h[p]] > const._FRIEND for p in [eighth_lord,tenth_lord,eleventh_lord]]) >=2
    md_checks.append(mc)
    """ Mercury, Venus, Jupiter in 2nd/3rd/11th house """
    mc = all([ _relative_house(p_to_h[p]) in [2,3,11] for p in [3,4,5]])
    """ All planets in middle four houses from lagna """
    mc = all([ _relative_house(p_to_h[p]) in [5,6,7,8] for p in range(7) ])
    md_checks.append(mc)
    return md_checks
def life_span_range(jd,place):
    """
        Alpayu = 0; Madhyayu = 1; Poornayu = 2
    """
    def _get_aayu(sign1, sign2):
        if sign1 in const.fixed_signs and sign2 in const.fixed_signs:
            return 0
        elif sign1 in const.movable_signs and sign2 in const.movable_signs:
            return 2
        elif sign1 in const.dual_signs and sign2 in const.dual_signs:
            return 1
        elif (sign1 in const.fixed_signs and sign2 in const.movable_signs) or \
             (sign1 in const.movable_signs and sign2 in const.fixed_signs):
            return 1
        elif (sign1 in const.dual_signs and sign2 in const.movable_signs) or \
             (sign1 in const.movable_signs and sign2 in const.dual_signs):
            return 0
        elif (sign1 in const.fixed_signs and sign2 in const.dual_signs) or \
             (sign1 in const.dual_signs and sign2 in const.fixed_signs):
            return 2
    planet_positions = charts.rasi_chart(jd, place)
    asc_house = planet_positions[0][1][0]; eigth_house = (asc_house+7)%12
    moon_house = planet_positions[2][1][0]
    lagna_lord = house.house_owner_from_planet_positions(planet_positions, asc_house)
    lagna_lord_house = planet_positions[lagna_lord+1][1][0]
    eighth_lord = house.house_owner_from_planet_positions(planet_positions, eigth_house)
    eighth_lord_house = planet_positions[eighth_lord+1][1][0]
    hora_lagna = drik.hora_lagna(jd,place)[0]
    _aayu_group = []
    _aayu_group.append(_get_aayu(lagna_lord_house, eighth_lord_house))
    _aayu_group.append(_get_aayu(asc_house, moon_house))
    _aayu_group.append(_get_aayu(asc_house, hora_lagna))
    import collections
    counter = collections.Counter(_aayu_group)
    #print(counter)
    if len(counter)==1: # All three same
        return counter.keys()[0]
    elif len(counter)==2:# two are same
        return max(counter,key=counter.get)
    else: # all 3 are different
        ret = _aayu_group[-1]
        if moon_house==asc_house or moon_house == (asc_house+6)%12:
            ret = _aayu_group[1]
        return ret
if __name__ == "__main__":
    horoscope_language = 'en' # """ Matplotlib charts available only English"""
    utils.set_language(horoscope_language)
    from jhora.horoscope.chart import charts
    from jhora.panchanga import drik
    dcf = 1; dob = (1996,12,7); tob = (10,34,0); place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    #dcf = 1; dob = (1964,11,16); tob = (4,30,0); place = drik.Place('Karamadi,India',11.2406,76.9601,5.5)
    #dcf = 1; dob = (1995,1,11); tob = (15,50,37); place = drik.Place('Royapuram,Tamil Nadu,India',13+6/50,80+17/60,5.5)
    jd = utils.julian_day_number(dob, tob)
    aa = life_span_range(jd,place); print(const.aayu_types[aa]);exit()
    print(drik._birthtime_rectification_nakshathra_suddhi(jd, place))
    print('_baladrishta_checks',_baladrishta_checks(jd,place))
    print('_alpayu_checks',_alpayu_checks(jd, place))
    print('_madhyayu_checks',_madhyayu_checks(jd, place))


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.horoscope.chart import house
def bhava_arudhas_from_planet_positions(planet_positions):
    """
        gives Bhava Arudhas for each house from the chart (A1=Arudha Lagna,A2.. A12=Upa Lagna)
        @param planet_positions: Planet Positions in the format: \
        [ [planet,[rasi,longitude]], [[,]].., [[,]]]
        @return bhava arudhas of houses. first element is for the first house from lagna and so on
    """
    """ V3.6.4 Below line is crucial. If Uranus/Netp/Pluto included wrong A1..A12 results
        So planet_positions restricted [:const._pp_count_upto_ketu] """
    planet_positions = planet_positions[:const._pp_count_upto_ketu]
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)
    asc_house = p_to_h[const._ascendant_symbol]
    houses = [(h+asc_house)%12 for h in range(12)]
    bhava_arudhas_of_houses =[]
    for h in houses:
        lord_of_the_house = house.house_owner_from_planet_positions(planet_positions, h, check_during_dhasa=False)
        house_of_the_lord = p_to_h[lord_of_the_house]
        signs_between_house_and_lord = utils.count_rasis(h,house_of_the_lord)
        bhava_arudha_of_house = (house_of_the_lord+signs_between_house_and_lord-1)%12
        signs_from_the_house = utils.count_rasis(h,bhava_arudha_of_house)#((bhava_arudha_of_house+1+12-h)%12)
        if signs_from_the_house in [1,7]:
            bhava_arudha_of_house = (bhava_arudha_of_house+10-1)%12
        bhava_arudhas_of_houses.append(bhava_arudha_of_house)
    return bhava_arudhas_of_houses
def bhava_arudhas(chart):
    """
        gives Bhava Arudhas for each house from the chart (A1=Arudha Lagna,A2.. A12=Upa Lagna)
        @param chart: Enter chart information in the following format. 
            For each house from Aries planet numbers separated by /
            ['0/1','2','','','3/4/5','','','6','L/7','','8','']
        @return bhava arudhas of houses. first element is for the first house from lagna and so on
    """
    h_to_p = chart[:]
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)
    asc_house = p_to_h[const._ascendant_symbol]
    houses = [(h+asc_house)%12 for h in range(12)]
    bhava_arudhas_of_houses =[]
    for h in houses:
        lord_of_the_house = house.house_owner(h_to_p, h) # V2.3.1
        house_of_the_lord = p_to_h[lord_of_the_house]
        signs_between_house_and_lord = utils.count_rasis(h,house_of_the_lord)
        bhava_arudha_of_house = (house_of_the_lord+signs_between_house_and_lord-1)%12
        signs_from_the_house = ((bhava_arudha_of_house+1+12-h)%12)
        if signs_from_the_house in [1,7]:
            bhava_arudha_of_house = (bhava_arudha_of_house+10-1)%12
        bhava_arudhas_of_houses.append(bhava_arudha_of_house)
    return bhava_arudhas_of_houses
def graha_arudhas_from_planet_positions(planet_positions):
    """
        gives Graha Arudhas for each planet from the planet positions
        @param planet_positions: Planet Positions in the format: \
        [ [planet,[rasi,longitude]], [[,]].., [[,]]]
        @return graha arudhas of planet. first element is for Sun, last element is for Ketu
    """
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)
    graha_arudhas_of_planets = []
    for p in range(const._planets_upto_ketu):
        house_of_the_planet = p_to_h[p]
        sign_owned_by_planet = const.house_lords_dict[p]
        if len(sign_owned_by_planet)>1:
            sign_owned_by_planet = house.stronger_rasi_from_planet_positions(planet_positions,sign_owned_by_planet[0],sign_owned_by_planet[1])
        else:
            sign_owned_by_planet = sign_owned_by_planet[0]
        count_to_strong = (sign_owned_by_planet+1+12-house_of_the_planet)%12
        count_to_arudha = (house_of_the_planet+2*(count_to_strong-1))%12
        count_from_house = (house_of_the_planet+12-count_to_arudha)%12
        if count_from_house in [0,6]:
            count_to_arudha = (count_to_arudha+9)%12
        graha_padha_of_planet = count_to_arudha
        graha_arudhas_of_planets.append(graha_padha_of_planet)
    return graha_arudhas_of_planets
def graha_arudhas(chart):
    """
        gives Graha Arudhas for each planet from the chart
        @param chart: Enter chart information in the following format. For each house from Aries planet numbers separated by /
            ['0/1','2','','','3/4/5','','','6','L/7','','8','']
        @return graha arudhas of planet. first element is for Sun, last element is for Ketu
    """
    h_to_p = chart[:]
    p_to_h = utils.get_planet_to_house_dict_from_chart(h_to_p)
    graha_arudhas_of_planets = []
    for p in range(const._planets_upto_ketu):
        house_of_the_planet = p_to_h[p]
        sign_owned_by_planet = const.house_lords_dict[p]
        if len(sign_owned_by_planet)>1:
            sign_owned_by_planet = house.stronger_rasi(h_to_p,sign_owned_by_planet[0],sign_owned_by_planet[1])
        else:
            sign_owned_by_planet = sign_owned_by_planet[0]
        count_to_strong = (sign_owned_by_planet+1+12-house_of_the_planet)%12
        count_to_arudha = (house_of_the_planet+2*(count_to_strong-1))%12
        count_from_house = (house_of_the_planet+12-count_to_arudha)%12
        if count_from_house in [0,6]:
            count_to_arudha = (count_to_arudha+9)%12
        graha_padha_of_planet = count_to_arudha
        graha_arudhas_of_planets.append(graha_padha_of_planet)
    return graha_arudhas_of_planets
    
if __name__ == "__main__":
    """
    from jhora.panchanga import drik
    dob = (1996,12,7); tob = (10,34,0); place = drik.Place('Chennai',13.0878,80.2785,5.5) 
    jd = utils.julian_day_number(dob, tob); dcf = 1
    from jhora.horoscope.chart import charts
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=dcf)
    ba = bhava_arudhas_from_planet_positions(planet_positions)
    print(ba); exit()
    """
    from jhora.tests import pvr_tests
    utils.set_language('en')
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.chapter_9_tests()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" To calculate Tajaka - Annual, monthly, sixty-hour, charts """
from jhora import const,utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house, strength
year_value = const.sidereal_year #const.tropical_year

kendras = lambda asc_house:[(asc_house+h-1)%12 for h in [1,4,7,10] ]
panaparas = lambda asc_house:[(asc_house+h-1)%12 for h in [2,5,8,11] ]
apoklimas = lambda asc_house:[(asc_house+h-1)%12 for h in [3,6,9,12] ]
""" Muntha house at x years after birth is xth house from Lagna. For example at 51th year, 50th house (i.e 4x12+2, 2nd house) from lagna """
muntha_house = lambda ascendant_house,years: (ascendant_house+years)%12
""" Aspects followed in Tajaka Analysis """
def trinal_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Trinal Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose trinal aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    aspected_houses = [(raasi+5-1)%12, (raasi+9-1)%12]
    aspected_planets = [ house_planet_dict[th].split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets    
def trinal_aspects_of_the_planet(house_planet_dict,planet):
    """
        Trinal Aspects of the planet (strong benefic aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose trinal aspects is sought
        @return: aspected raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return trinal_aspects_of_the_raasi(house_planet_dict,planet_house)
def sextile_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Sextile Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose sextile aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    aspected_houses = [(raasi+3-1)%12, (raasi+11-1)%12]
    aspected_planets = [ house_planet_dict[th].split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets
def sextile_aspects_of_the_planet(house_planet_dict,planet):
    """
        Sextile Aspects of the planet (weak benefic aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose sextile aspects is sought
        @return: aspected raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return sextile_aspects_of_the_raasi(house_planet_dict,planet_house)
def square_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Square Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose square aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    aspected_houses = [(raasi+4-1)%12, (raasi+10-1)%12]
    aspected_planets = [ house_planet_dict[th].split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets
def square_aspects_of_the_planet(house_planet_dict,planet):
    """
        Square Aspects of the planet (weak maleefic aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose sextile aspects is sought
        @return: aspected raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return square_aspects_of_the_raasi(house_planet_dict,planet_house)
def benefic_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Benefic Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose benefic aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    trh,trp = trinal_aspects_of_the_raasi(house_planet_dict, raasi)
    sqh,sqp = sextile_aspects_of_the_raasi(house_planet_dict, raasi)
    bah = trh + sqh
    bap = trp+sqp#sum(trp+sqp,[])
    return bah,bap
def benefic_aspects_of_the_planet(house_planet_dict,planet):
    """
        Benefic Aspects of the planet (weak maleefic aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose benefic aspects is sought
        @return: aspected raasis, aspected planets - as a list 
    """
    trh,trp = trinal_aspects_of_the_planet(house_planet_dict, planet)
    sqh,sqp = sextile_aspects_of_the_planet(house_planet_dict, planet)
    bah = trh + sqh
    bap = trp+sqp#sum(trp+sqp,[])
    return bah,bap
def planet_has_benefic_aspect_on_house(house_planet_dict,planet,house):
    """
        Return True/False if planet has benefic on a house
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose benefic aspects is sought
        @param house:House index whose benefic aspects is sought
        @return: True/False 
    """
    bh,_ = benefic_aspects_of_the_planet(house_planet_dict, planet)
    return house in bh
def semi_sextile_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Semi sextile Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose semi sextile aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    aspected_houses = [(raasi+2-1)%12, (raasi+12-1)%12]
    aspected_planets = [ house_planet_dict[th].split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets
def semi_sextile_aspects_of_the_planet(house_planet_dict,planet):
    """
        Semi sextile Aspects of the planet (Neutral aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:palnet index whose semi sextile aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return semi_sextile_aspects_of_the_raasi(house_planet_dict,planet_house)
def neutral_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Neutral Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose neutral aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    return semi_sextile_aspects_of_the_raasi(house_planet_dict,raasi)
def neutral_aspects_of_the_planet(house_planet_dict,planet):
    """
        Neutral Aspects of the Planet
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:planet index whose neutral aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    return semi_sextile_aspects_of_the_planet(house_planet_dict,planet)
def opposition_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Opposition Aspects of the Raasi (Strong Malefic Aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose opposition aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    aspected_houses = [(raasi+7-1)%12]
    aspected_planets = [ house_planet_dict[th].split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets
def opposition_aspects_of_the_planet(house_planet_dict,planet):
    """
        Opposition Aspects of the Planet (Strong Malefic Aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose opposition aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return opposition_aspects_of_the_raasi(house_planet_dict,planet_house)
def conjunction_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Conjunction Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose conjunction aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    p_to_h = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet = list(p_to_h.keys())[list(p_to_h.values()).index(raasi)]
    aspected_houses = [raasi]
    aspected_planets = [ house_planet_dict[th].replace(str(planet),'').split('/') for th in aspected_houses if house_planet_dict[th]!='']
    aspected_planets = sum(aspected_planets,[])
    aspected_planets = [ap for ap in aspected_planets if ap.strip() != '']
    return aspected_houses,aspected_planets
def conjunction_aspects_of_the_planet(house_planet_dict,planet):
    """
        Conjunction Aspects of the Planet (strong malefic aspect)
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose conjunction aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_planet_dict)
    planet_house = planet_to_house_dict[planet]
    return conjunction_aspects_of_the_raasi(house_planet_dict,planet_house)
def malefic_aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Malefic Aspects of the Raasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose malefic aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    sqh,sqp = square_aspects_of_the_raasi(house_planet_dict, raasi)
    coh,cop = conjunction_aspects_of_the_raasi(house_planet_dict, raasi)
    oph,opp = opposition_aspects_of_the_raasi(house_planet_dict, raasi)
    mah = sqh + coh + oph
    mapp = sqp+cop+opp#sum(sqp+cop+opp,[])
    return mah,mapp
def malefic_aspects_of_the_planet(house_planet_dict,planet):
    """
        Malefic Aspects of the Planet
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose malefic aspects is sought
        @return: aspected Raasis, aspected planets - as a list 
    """
    sqh,sqp = square_aspects_of_the_planet(house_planet_dict, planet)
    coh,cop = conjunction_aspects_of_the_planet(house_planet_dict, planet)
    oph,opp = opposition_aspects_of_the_planet(house_planet_dict, planet)
    mah = sqh + coh + oph
    mapp = sqp+cop+opp#sum(sqp+cop+opp,[])
    return mah,mapp
def planets_have_benefic_aspects(house_planet_dict,planet1,planet2):
    """
        Return True/False if planet1 and planet2 have benefic aspects on each other
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet1:Planet1 index whose benefic aspects is sought
        @param planet1:Planet2 index whose benefic aspects is sought
        @return: True/False if both planets have benefic aspect on each other
    """
    return str(planet2) in benefic_aspects_of_the_planet(house_planet_dict, planet1)[1]
def planets_have_malefic_aspects(house_planet_dict,planet1,planet2):
    """
        Return True/False if planet1 and planet2 have malefic aspects on each other
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet1:Planet1 index whose malefic aspects is sought
        @param planet1:Planet2 index whose malefic aspects is sought
        @return: True/False if both planets have malefic aspect on each other
    """
    return str(planet2) in malefic_aspects_of_the_planet(house_planet_dict, planet1)[1]
def planets_have_neutral_aspects(house_planet_dict,planet1,planet2):
    """
        Return True/False if planet1 and planet2 have neutral aspects on each other
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet1:Planet1 index whose neutral aspects is sought
        @param planet1:Planet2 index whose neutral aspects is sought
        @return: True/False if both planets have neutral aspect on each other
    """
    return str(planet2) in neutral_aspects_of_the_planet(house_planet_dict, planet1)[1]
def planet_aspects_from_chart(chart):
    planet_aspects = {k:[] for k in range(9)}
    for planet1 in range(9):
        for planet2 in range(9):
            if planets_have_aspects(chart, planet1, planet2):
                planet_aspects[planet1].append(planet2)
    return planet_aspects
    
def planets_have_aspects(house_planet_dict,planet1,planet2):
    """
        Return True/False if planet1 and planet2 have ANY aspects on each other
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet1:Planet1 index whose aspects is sought
        @param planet1:Planet2 index whose aspects is sought
        @return: True/False if both planets have ANY aspect on each other
    """
    chk1 = planets_have_benefic_aspects(house_planet_dict,planet1,planet2)
    chk2 = planets_have_malefic_aspects(house_planet_dict,planet1,planet2)
    chk3 = planets_have_neutral_aspects(house_planet_dict,planet1,planet2)
    return chk1 or chk2 or chk3
def planet_has_malefic_aspect_on_house(house_planet_dict,planet,house):
    """
        Return True/False if planet has malefic aspect on the house
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose malefic aspects is sought
        @param house:House index whose malefic aspects is sought
        @return: True/False if planet has malefic aspect on the house
    """
    bh,_ = malefic_aspects_of_the_planet(house_planet_dict, planet)
    return house in bh
def aspects_of_the_raasi(house_planet_dict,raasi):
    """
        Return benefic, malefic and neutral aspected of the rasi
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param raasi:Raasi index whose malefic aspects is sought
        @return aspected houses, aspected planets
    """
    bah,bap = benefic_aspects_of_the_raasi(house_planet_dict, raasi)
    mah,map = malefic_aspects_of_the_raasi(house_planet_dict, raasi)
    nah,nap = neutral_aspects_of_the_raasi(house_planet_dict, raasi)
    ah = bah + mah + nah
    ap = bap + map + nap
    #ap = utils.flatten_list(ap)
    return ah,ap
def aspects_of_the_planet(house_planet_dict,planet):
    """
        Return benefic, malefic and neutral aspects of the planet
        @param house_to_planet_dict: list of raasi with planet ids in them
          Example: ['','','','','2','7','1/5','0','3/4','L','','6/8'] 1st element is Aries and last is Pisces
        @param planet:Planet index whose malefic aspects is sought
        @return aspected houses, aspected planets
    """
    bah,bap = benefic_aspects_of_the_planet(house_planet_dict, planet)
    mah,map = malefic_aspects_of_the_planet(house_planet_dict, planet)
    nah,nap = neutral_aspects_of_the_planet(house_planet_dict, planet)
    ah = bah + mah + nah
    ap = bap + map + nap
    #ap = utils.flatten_list(ap)
    return ah,ap
def varsha_pravesh(jd_at_dob,place,divisional_chart_factor=1,years=1):
    return annual_chart(jd_at_dob,place,divisional_chart_factor,years)
def annual_chart_approximate(dob,tob,place,divisional_chart_factor=1,years=1):
    """
        (1) Find the birthday as per western calendar in the required year.
        (2) Find the years completed. Find the corresponding days, hours, minutes and
        seconds from Table 71. If the age is not in the list, express it as a sum of entries
        found in the table and add their values. For example, suppose someone finished
        46 years. Then add the values given for 40 years and 6 years.
        (3) Add the days found above to the weekday77 of birth and find the resulting
        weekday. Find the nearest date to the birthday found in (1) that falls on this
        weekday. A time equal to the birthtime on this date is taken as a reference. 
        (4) Add the hours, minutes and seconds found in (2) to the reference date and time
        found in (3). The result is the date and time of the commencement of new year.
        (5) Find the planetary positions, lagna etc at this time for the longitude and latitude
        of the birthplace.   
    """
    jd_at_dob = utils.julian_day_number(dob, tob)
    natal_chart = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, divisional_chart_factor=divisional_chart_factor)
    natal_solar_long = natal_chart[1][1][0]*30+natal_chart[1][1][1]
    jd_years = drik.next_annual_solar_date_approximate(dob, tob, years)
    yn,mn,dn,fhn = utils.jd_to_gregorian(jd_years)
    cht = _get_tajaka_chart(jd_years,place,divisional_chart_factor)
    years_solar_long = cht[1][1][0]*30+cht[1][1][1]
    solar_long_diff = natal_solar_long-years_solar_long
    solar_long_correction = solar_long_diff*year_value/360.0
    jd_years_corrected = jd_years+solar_long_correction
    #print('natal_solar_long',utils.to_dms(natal_solar_long,is_lat_long='plong'),
    #      'years_solar_long',utils.to_dms(years_solar_long,is_lat_long='plong'),
    #      'solar_long_diff',utils.to_dms(solar_long_diff,is_lat_long='plong'))
    #print('solar_long_correction',utils.to_dms(solar_long_correction*24,as_string=True),jd_years,jd_years_corrected)
    yc,mc,dc,fhc = utils.jd_to_gregorian(jd_years_corrected)
    #fhn = utils.to_dms(fhn)
    fhc = utils.to_dms(fhc)
    #print('jd_years',(yn,mn,dn,fhn),'jd_years_corrected',(yc,mc,dc,fhc))
    cht = _get_tajaka_chart(jd_years_corrected,place,divisional_chart_factor)
    return cht,[(yc,mc,dc),fhc]
def annual_chart(jd_at_dob,place,divisional_chart_factor=1,years=1):
    """
        Also can be called using:
            varsha_pravesh (jd_at_dob, place, divisional_chart_factor=1, years=1)
        Create Tajaka Annual Chart. Tajaka annual chart is chart for one year at "years" from date of birth/time of birth
        @param jd_at_dob: Julian Day nummber at date/time of birth
            Note: You can use swe.julday(dob_year,dob_month,dob_day,tob_hour+tob_minutes/60.0+tob_seconds/3600.0) to get
        @param place: should be a struct os drik.Place (place,latitude,longitude,time_sone_factor)
        @param divisional_chart_factor: 1=Rasi, 2=Hota, 9=navamsa etc. See drik.division_chart_factors for details
        @param years: number of years after dob the dhasa varga chart is sought 
        @return: Tjaka annual dhasa varga chart as list of planets
    """
    #jd_years = jd_at_dob + years*year_value_year
    jd_years = drik.next_solar_date(jd_at_dob, place, years=years) #, months, sixty_hours)
    y,m,d,fh = utils.jd_to_gregorian(jd_years)
    cht = _get_tajaka_chart(jd_years,place,divisional_chart_factor)
    return cht,[(y,m,d),utils.to_dms(fh)]
def _get_tajaka_chart(jd,place,divisional_chart_factor):
    return charts.divisional_chart(jd,place,divisional_chart_factor=divisional_chart_factor)
def maasa_pravesh(jd_at_dob,place,divisional_chart_factor=1,years=1,months=1):
    return monthly_chart(jd_at_dob,place,divisional_chart_factor,years,months)
def monthly_chart(jd_at_dob,place,divisional_chart_factor=1,years=1,months=1):
    """
        Also can be called using:
            maasa_pravesh (jd_at_dob, place, divisional_chart_factor=1, years=1, months=1)
        Create Tajaka monthly Chart. Tajaka monthly chart is chart for nth month at "years" from date of birth/time of birth
        @param jd_at_dob: Julian Day nummber at date/time of birth
            Note: You can use swe.julday(dob_year,dob_month,dob_day,tob_hour+tob_minutes/60.0+tob_seconds/3600.0) to get
        @param place: should be a struct os drik.Place (place,latitude,longitude,time_sone_factor)
        @param divisional_chart_factor: 1=Rasi, 2=Hota, 9=navamsa etc. See drik.division_chart_factors for details
        @param years: number of years after dob the dhasa varga chart is sought 
        @param months: number of month after years after dob the dhasa varga chart is sought 
        @return: Tjaka annual dhasa varga chart as list of planets
    """
    if months<1 or months > 12:
        months = 1
    #jd_years = jd_at_dob + (years + (months/12.0))*year_value
    jd_years = drik.next_solar_date(jd_at_dob, place, years, months)#, sixty_hours)
    y,m,d,fh = utils.jd_to_gregorian(jd_years)
    cht = _get_tajaka_chart(jd_years,place,divisional_chart_factor)
    return cht,[(y,m,d),utils.to_dms(fh)]
def sixty_hour_chart(jd_at_dob,place,divisional_chart_factor=1,years=1,months=1,sixty_hour_count=1):
    """
        Create Tajaka sizty hour Chart.
        @param jd_at_dob: Julian Day nummber at date/time of birth
            Note: You can use swe.julday(dob_year,dob_month,dob_day,tob_hour+tob_minutes/60.0+tob_seconds/3600.0) to get
        @param place: should be a struct os drik.Place (place,latitude,longitude,time_sone_factor)
        @param divisional_chart_factor: 1=Rasi, 2=Hota, 9=navamsa etc. See drik.division_chart_factors for details
        @param years: number of years after dob the dhasa varga chart is sought 
        @param months: number of month after years after dob the dhasa varga chart is sought 
        @param sizty_hour_count: number of 2.5 days in the tajaka month after years after dob the dhasa varga chart is sought 
        @return: Tjaka annual dhasa varga chart as list of planets
    """
    jd_years = drik.next_solar_date(jd_at_dob, place, years, months, sixty_hour_count)
    y,m,d,fh = utils.jd_to_gregorian(jd_years)
    #jd_years = jd_at_dob + (years + (months/12.0)+(sixty_hour_count/144.0))*year_value
    cht = _get_tajaka_chart(jd_years,place,divisional_chart_factor)
    return cht,[(y,m,d),utils.to_dms(fh)]
def _get_lord_candidates(planet_positions,years_from_dob,natal_lagna_house,night_time_birth):
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    tajaka_chart_h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(tajaka_chart_p_to_h)
    " Rule-1 Sun-Sign's  or Moon-Sign's Lord?"
    candidates = []
    if night_time_birth:
        candidates.append(house.house_owner_from_planet_positions(planet_positions,tajaka_chart_p_to_h[1]))
    else:
        candidates.append(house.house_owner_from_planet_positions(planet_positions,tajaka_chart_p_to_h[0]))
    candidates.append(house.house_owner_from_planet_positions(planet_positions,natal_lagna_house))
    asc_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    m_house = muntha_house(asc_house,years_from_dob)
    candidates.append(house.house_owner_from_planet_positions(planet_positions,m_house))
    candidates.append(house.house_owner_from_planet_positions(planet_positions,asc_house))
    if night_time_birth:
        candidates.append(const.tri_rasi_nighttime_lords[asc_house]) 
    else:
        candidates.append(const.tri_rasi_daytime_lords[asc_house])
    ap = house.aspected_planets_of_the_raasi(tajaka_chart_h_to_p, asc_house)
    candidates = list(sorted(set(candidates),key=candidates.index)) # Remove duplicates and keep order same  
    return candidates
def _get_the_lord_of_tajaka_chart(jd, place,candidates):
    rasi_chart = charts.divisional_chart(jd, place,divisional_chart_factor=1)
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    tajaka_chart_h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(tajaka_chart_p_to_h)
    asc_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    #print('candidates',candidates)
    asc_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    candidates_shortlisted = [candidate for candidate in candidates if planet_has_benefic_aspect_on_house(tajaka_chart_h_to_p, candidate, asc_house)]
    #print('candidates_shortlisted based on benefic aspects',candidates_shortlisted)
    if len(candidates_shortlisted) == 1:
        lord_of_the_year = candidates_shortlisted[0] # Lord of the Year
        return lord_of_the_year
    if len(candidates_shortlisted) == 0:
        candidates_shortlisted = [candidate for candidate in candidates if planet_has_malefic_aspect_on_house(tajaka_chart_h_to_p, candidate, asc_house)]
    #print('candidates_shortlisted based on malefic aspects',candidates_shortlisted)
    if len(candidates_shortlisted) == 1:
        lord_of_the_year = candidates_shortlisted[0] # Lord of the Year 
        return lord_of_the_year
    " No or more than one candidate benefic or malefic - so let us check highest panchavargeeya bala"
    
    pvb = strength.pancha_vargeeya_bala(jd, place)
    pvbc = [pvb[candidate] for candidate in candidates]
    #pvbcl = pvbc[lord_of_the_year]
    pvb_max = max(pvbc)
    if pvb_max > const.pancha_vargeeya_bala_strength_threshold:
        lord_of_the_year = pvbc.index(pvb_max)
        #print('Lord of year as per pancha veerya bala (max) ',lord_of_the_year,'is',pvb_max)
        return lord_of_the_year
    "rasi occupied by Sun or Moon in the annual chart - candidate-1"
    lord_of_the_year = candidates[0]
    #print('Lord of the year based on new year start',lord_of_the_year)
    return lord_of_the_year    
def lord_of_the_year(jd_at_dob,place,years_from_dob):#,night_time_birth=False):
    """
        Get the Lord of the year/annual chart - Get natal lagna house from birth chart
        @param jd_at_dob: Julin Day Number for the date of birth
        @param place: pancha.Place Struct Place('place name',latitude,longitude,timezone)
        @param years_from_dob:# of years from date of birth.
            For example if Lord of the 25th year from DOB, then years_from_dob=25
        @return:  Lord of the year (planet index) [0..6]
            Note: Rahu/Ketu dont form lord of the year
    """
    rasi_chart = charts.divisional_chart(jd_at_dob, place,divisional_chart_factor=1)
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    tajaka_chart_h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(tajaka_chart_p_to_h)
    natal_lagna_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    #print('natal_lagna_house',natal_lagna_house)
    " Get annual chart "
    jd_at_years = jd_at_dob + years_from_dob*year_value
    tob_hrs = drik.jd_to_gregorian(jd_at_years)[3]
    sunrise = utils.from_dms_str_to_dms(drik.sunrise(jd_at_years, place)[1]) #2.0.3
    sunrise_hrs = sunrise[0]+sunrise[1]/60.0+sunrise[2]/3600.0
    sunset = utils.from_dms_str_to_dms(drik.sunset(jd_at_years, place)[1]) # 2.0.3
    sunset_hrs = sunset[0]+sunset[1]/60.0+sunset[2]/3600.0
    night_time_birth = tob_hrs > sunset_hrs or tob_hrs < sunrise_hrs
    #print('night_time_birth',night_time_birth,sunrise_hrs,tob_hrs,sunset_hrs)
    rasi_chart = charts.divisional_chart(jd_at_years, place,divisional_chart_factor=1)
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    tajaka_chart_h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(tajaka_chart_p_to_h)
    asc_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    #print('asc_house',asc_house)
    candidates = _get_lord_candidates(rasi_chart,years_from_dob,natal_lagna_house,night_time_birth)
    return _get_the_lord_of_tajaka_chart(jd_at_years, place,candidates)
def lord_of_the_month(jd_at_dob,place,years_from_dob,months_from_dob):
    """
        Get the Lord of the monthly chart - Get natal lagna house from birth chart
        @param jd_at_dob: Julin Day Number for the date of birth
        @param place: pancha.Place Struct Place('place name',latitude,longitude,timezone)
        @param years_from_dob:# of years from date of birth.
            For example if Lord of the 25th year from DOB, then years_from_dob=25
        @param months_from_dob:# of months on the years_from_dob.
            For example if 10th month of 25th year from DOB, then months_from_dob=10
        @return:  Lord of the month (planet index) [0..6]
            Note: Rahu/Ketu dont form lord of the month
    """
    rasi_chart = charts.divisional_chart(jd_at_dob, place,divisional_chart_factor=1)
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    natal_lagna_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    #print('natal_lagna_house',natal_lagna_house)
    lord_of_year = lord_of_the_year(jd_at_dob, place, years_from_dob)
    jd_at_years = jd_at_dob + (years_from_dob+months_from_dob/12.0)*year_value
    
    tob_hrs = drik.jd_to_gregorian(jd_at_years)[3]
    sunrise = utils.from_dms_str_to_dms(drik.sunrise(jd_at_years, place)[1]) #2.0.3
    sunrise_hrs = sunrise[0]+sunrise[1]/60.0+sunrise[2]/3600.0
    sunset = utils.from_dms_str_to_dms(drik.sunset(jd_at_years, place)[1]) #2.0.3
    sunset_hrs = sunset[0]+sunset[1]/60.0+sunset[2]/3600.0
    night_time_birth = tob_hrs > sunset_hrs or tob_hrs < sunrise_hrs
    #print('night_time_birth',night_time_birth,sunrise_hrs,tob_hrs,sunset_hrs)
    rasi_chart = charts.divisional_chart(jd_at_years, place,divisional_chart_factor=1)
    tajaka_chart_p_to_h = utils.get_planet_house_dictionary_from_planet_positions(rasi_chart)
    tajaka_chart_h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(tajaka_chart_p_to_h)
    asc_house = tajaka_chart_p_to_h[const._ascendant_symbol]
    #print('asc_house',asc_house)
    candidates = _get_lord_candidates(rasi_chart,years_from_dob,natal_lagna_house,night_time_birth)
    candidates += [lord_of_year]
    return _get_the_lord_of_tajaka_chart(jd_at_years, place,candidates)
def both_planets_within_their_deeptamsa(planet_positions,planet1,planet2):
    """
        Check if two planets are within their deeptamsa
        @param planet_positions: Planet Positions in the format [[planet,(raasi,longitude)],...]
        @param planet1: First planet index
        @param planet2: Second planet index comapred against
        @return: True/False, Ithasala Type
            Ithasala Type: 1. Varthamaana ithasala, 2. Bhavishya ithasala, 3. Poorna ithasala
    """
    planet1_long_within_raasi = planet_positions[planet1+1][1][1]
    planet2_long_within_raasi = planet_positions[planet2+1][1][1]
    ithasala_type = None
    planet1_deeptamsa_start,planet1_deeptamsa_end = utils.deeptaamsa_range_of_planet(planet1,planet1_long_within_raasi)
    planet2_deeptamsa_start,planet2_deeptamsa_end = utils.deeptaamsa_range_of_planet(planet2,planet2_long_within_raasi)
    chk1 = planet1_long_within_raasi >= planet2_deeptamsa_start and planet1_long_within_raasi <= planet2_deeptamsa_end
    chk1_1 = False
    if not chk1: #Check for bhavishya ithasala
        chk1_1 = abs(planet1_long_within_raasi-planet2_deeptamsa_start)<=1.0 or abs(planet1_long_within_raasi - planet2_deeptamsa_end)<=1.0
    #print('deeptamsa test',planet1,'in house',planet1_house,planet2_deeptamsa_start,planet2_deeptamsa_end,'planet1_long_within_raasi',planet1_long_within_raasi,'chk1',chk1)
    chk2 = planet2_long_within_raasi >= planet1_deeptamsa_start and planet2_long_within_raasi <= planet1_deeptamsa_end
    chk2_1 = False
    if not chk2: #Check for bhavishya ithasala
        chk2_1 = abs(planet2_long_within_raasi-planet1_deeptamsa_start)<=1.0 or abs(planet2_long_within_raasi - planet1_deeptamsa_end)<=1.0
    #print('deeptamsa test',planet2,'in house',planet2_house,planet1_deeptamsa_start,planet1_deeptamsa_end,'planet1_long_within_raasi',planet2_long_within_raasi,'chk2',chk2)
    ithasala = chk1 and chk2
    if ithasala:
        ithasala_type = 1 # Varthamaana
    elif (chk1 and chk2_1) or (chk2 and chk1_1):
        ithasala_type = 3 # Bhavishya
        ithasala = True
    if abs(planet1_long_within_raasi-planet2_long_within_raasi) <= 1.0:
        ithasala_type = 2 # Poorna ithasala
    return ithasala,ithasala_type
def both_planets_approaching(planet_positions,planet1,planet2):
    """
        Check if two planets are approaching each other
        NOTE: TODO: Check if planets in retrograde so they are moving away
        @param planet_positions: Planet Positions in the format [[planet,(raasi,longitude)],...]
        @param planet1: First planet index
        @param planet2: Second planet index comapred against
        @return: True/False
    """
    planet1_long_within_raasi = planet_positions[planet1+1][1][1]
    planet2_long_within_raasi = planet_positions[planet2+1][1][1]
    " Check speed of their planets and their advancement within raasi - faster planet less advanced"
    faster_planet = planet1
    if const.order_of_planets_by_speed.index(planet1) < const.order_of_planets_by_speed.index(planet2):
        faster_planet = planet2
    advanced_planet = planet1
    if planet1_long_within_raasi < planet2_long_within_raasi:
        advanced_planet = planet2
    #retrograde_planets = charts.planets_in_retrograde(planet_positions)
    #print('retrograde_planets',retrograde_planets,'faster planet',faster_planet,'advanced planet',advanced_planet)
    chk3_1 = (advanced_planet==planet2) and (faster_planet==planet1)
    chk3_2 = (advanced_planet==planet1) and (faster_planet==planet2)
    chk3 = chk3_1 or chk3_2
    #print('deeptamsa test',planet1,'is faster?',chk3_1,' OR ',planet2,'is faster?',chk3_2,'chk3',chk3)
    return chk3    
    """
    TODO: Implement retrogression in this ithasala yoga calculation
    # Retrograde check - following combinations for approaching
    # L is faster/forward R is slower/forward/backward
    # L is slower/forward R is faster backward
    # L is slower backward and R is faster backward
    chk4_1 = (planet1 not in retrograde_planets) and (planet1 == faster_planet)
    print('(planet1 not in retrograde_planets) and (planet1 == faster_planet)',chk4_1)
    chk4_2 = (planet2==faster_planet and planet2 in retrograde_planets)
    print('(planet2==faster_planet and planet2 in retrograde_planets)',chk4_2)
    chk4 = chk4_1 or chk4_2
    """
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.chapter_27_tests()
    


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
from jhora.horoscope.chart import charts, house
from jhora import utils, const
from jhora.panchanga import drik
_lang_path = const._LANGUAGE_PATH

def get_prediction_resources(language='en'):
    """
        get resources from prediction_msgs_<lang>.txt
        @param language: Two letter language code. en, hi, ka, ta, te
        @return json strings from the resource file as dictionary 
    """
    json_file = _lang_path + const._DEFAULT_PREDICTION_JSON_FILE_PREFIX+language+'.json'
    f = open(json_file,"r",encoding="utf-8")
    msgs = json.load(f)
    return msgs

def _get_general_lagna_rasi_prediction(jd,place,prediction_msgs,language=const._DEFAULT_LANGUAGE):
    janma_rasi = drik.raasi(jd, place)[0]-1
    results = {}
    source_count = 2
    for s in range(source_count):
        ks = utils.resource_strings['janma_rasi_str']+'_'+str(s+1)
        results[ks] = "<html><b>"+utils.resource_strings['general_prediction_str']+"</b><br>"
        #results[ks] += "<b>"+prediction_msgs['general_prediction_caution']+"</b><br>"
        results[ks] += "<b>"+prediction_msgs['janma_raasi_'+str(s+1)]['source']+"</b><br>"
        pdict = prediction_msgs['janma_raasi_'+str(s+1)][str(janma_rasi+1)]
        for k,v in pdict.items():
            results[ks] += "<b>"+k+"</b><br>"+v+"<br>"
    return results
def _get_planets_in_houses_prediction(planet_positions,prediction_msgs):
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    lagna_house = p_to_h['L']
    ks = utils.resource_strings['planets_str']
    results = {}
    results[ks] = "<html>"#<b>"+ks+"</b><br>"
    #results[ks] += "<b>"+utils.resource_strings['general_prediction_caution']+"</b><br>"
    planet_msgs = prediction_msgs['planets_in_houses']
    #print('planet msgs',planet_msgs)
    for planet in [*range(9)]:
        planet_house = house.get_relative_house_of_planet(lagna_house,p_to_h[planet])
        pl_msg = planet_msgs[str(planet_house)][planet]
        #print(planet,planet_house,pl_msg)
        key = utils.PLANET_NAMES[planet]+'-'+utils.resource_strings['house_str']+'#'+str(planet_house)+":"
        results[ks] += "<b>"+key+"</b><br>"+pl_msg+"<br>"
    return results
def _get_lords_in_houses_prediction(planet_positions,prediction_msgs):
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    lagna_house = p_to_h['L']
    ks = utils.resource_strings['houses_str']
    results = {}
    results[ks] = "<html>"#<b>"+ks+"</b><br>"
    #results[ks] += "<b>"+utils.resource_strings['general_prediction_caution']+"</b><br>"
    planet_msgs = prediction_msgs['lord_of_a_house_joining_lord_of_another_house']
    #print('planet msgs',planet_msgs)
    for h in [*range(12)]:
        lord = const._house_owners_list[(h+lagna_house)%12]
        house_of_lord = house.get_relative_house_of_planet(lagna_house,p_to_h[lord])
        key = "Lord of House#"+str(h+1)+" in house#"+str(house_of_lord)
        #print('key',key)
        pl_msg = planet_msgs[str(h+1)][house_of_lord-1]
        results[ks] += "<b>"+key+"</b><br>"+pl_msg+"<br>"
    return results
def get_prediction_details(jd_at_dob,place,language=const._DEFAULT_LANGUAGE):
    prediction_msgs = get_prediction_resources(language=language)
    #print('prediction keys',prediction_msgs.keys())
    results = {}
    planet_positions = charts.rasi_chart(jd_at_dob, place)
    results1 = _get_general_lagna_rasi_prediction(jd_at_dob,place,prediction_msgs,language=language)
    results.update(results1)
    results2 = _get_planets_in_houses_prediction(planet_positions,prediction_msgs)
    results.update(results2)
    results3 = _get_lords_in_houses_prediction(planet_positions,prediction_msgs)
    results.update(results3)
    return results
if __name__ == "__main__":
    lang = 'te'
    utils.set_language(lang)
    res = utils.resource_strings
    from jhora.horoscope.chart import charts
    from jhora.panchanga import drik
    dob = (1996,12,7); tob = (10,34,0); jd_at_dob = utils.julian_day_number(dob, tob)
    place_as_tuple = drik.Place('Chennai, India',13.0878,80.2785,5.5)
    planet_positions = charts.rasi_chart(jd_at_dob, place_as_tuple)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    print(h_to_p)
    #prediction_msgs = get_prediction_resources(language=lang)
    ks = get_prediction_details(jd_at_dob,place_as_tuple, language=lang)
    print(ks)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora.horoscope.chart import house
from jhora import utils
"""
    DO NOT USE THIS YET. UNDER TESTING...
    Based on Book: Prediction Secrets Naadi Astrology - A research work based on Rao's system - by Sri Satyanarayana Naik
"""
def _check_yoga_1(planet_positions:list=None,gender:int=0):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    """ Yoga-1 / Yoga-7 - check if marriage planet of gender aspecting Jupiter or Saturn """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    planet = 5 # Venus
    if gender==1: planet = 2 # Mars
    aspected_planets = house.aspected_planets_of_the_planet(house_to_planet_list, planet)
    y1 = any([p1 in aspected_planets for p1 in [4,6]])
    y1msg = "Obstacles in marriage, if married, disputes with spouse"
    if y1: y1msg = "Marriage will happen"
    print('4/6 in aspected planets graha drishti of planet',planet,aspected_planets,y1,y1msg)
    return (y1,y1msg)
def _check_yoga_2(planet_positions:list=None,gender:int=0):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    """ yoga-2 / yoga - 8 / Yoga-3/Yoga-9  Early/late marriage"""
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    planet = 5 # Venus
    if gender==1: planet = 2 # Mars
    aspected_planets = house.aspected_planets_of_the_planet(house_to_planet_list, planet)
    y2 = 4 in aspected_planets
    if y2:
        y2msg = "Early Marriage"
        print('yoga 2/8 Is Jupiter Aspecting',planet,aspected_planets,y2,y2msg)
    else:
        y2 = 6 in aspected_planets
        y2msg = "Late Marriage" if y2 else "Neither Early/Late Marriage"
        print('yoga 2/8 Is Saturn Aspecting',planet,aspected_planets,y2,y2msg)
    return (y2,y2msg)
def _check_yoga_3(planet_positions:list=None,gender:int=0):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    """ Yoga-4 / obstacles in marriage, If married disputes with spouse """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    planet = 5 # Venus
    if gender==1: planet = 2 # Mars
    aspected_planets = house.aspected_planets_of_the_planet(house_to_planet_list, 8)
    y3msg = ""; y3 = planet in aspected_planets
    if y3: y3msg = "Obstacles in marriage, If married disputes with spouse."
    print('yoga 4/10 Is Kethu Aspecting',planet,aspected_planets,y3,y3msg)
    return (y3,y3msg)
    
def _check_yoga_4(planet_positions:list=None,gender:int=0):
    house_to_planet_list = utils.get_house_planet_list_from_planet_positions(planet_positions)
    """ Yoga-5 / Marriage delayed or if married, will not like the spouse. """
    planet_to_house_dict = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    planet = 5 # Venus
    if gender==1: planet = 2 # Mars
    aspected_planets = house.aspected_planets_of_the_planet(house_to_planet_list, 7)
    y4msg = ""; y4 = planet in aspected_planets
    if y4: y4msg = "Marriage delayed or if married, will not like the spouse."
    print('yoga 6/16 Is Rahu Aspecting',planet,aspected_planets,y4,y4msg)
    return (y4,y4msg)
def _check_marriage_yogas(planet_positions:list=None,gender:int=0):
    mc = []
    mc.append(_check_yoga_1(planet_positions, gender))
    mc.append(_check_yoga_2(planet_positions, gender))
    mc.append(_check_yoga_3(planet_positions, gender))
    mc.append(_check_yoga_4(planet_positions, gender))
    return mc

if __name__ == "__main__":
    horoscope_language = 'en' # """ Matplotlib charts available only English"""
    utils.set_language(horoscope_language)
    gender = 1 # Female
    #"""
    from jhora.horoscope.chart import charts
    from jhora.panchanga import drik
    dcf = 1
    pp = charts.divisional_chart(utils.julian_day_number((1996,12,7), (10,34,0)), drik.Place('Chennai,India',13.0878,80.2785,5.5),divisional_chart_factor=dcf)
    h_to_p = utils.get_house_planet_list_from_planet_positions(pp)
    print(h_to_p)
    """
    h_to_p_1 = ['5/2/3/0','','','L/4','6/7','','','1','','','8','']
    h_to_p_2 = ['','','8','L/4','','1','','','7','6','2/3/5','0']
    #gender = 0
    h_to_p_3 = ['L/2','1','3','0/5','7','','','','4/6','','8','']
    """
    print(_check_marriage_yogas(pp,gender))
    


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora.panchanga import drik
from jhora import const,utils
from jhora.horoscope.chart import house, charts

def tri_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    asc_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=mixed_dvf)
    gulika_long = gulika[0]*30+gulika[1]
    _tri_sphuta = (moon_long+asc_long+gulika_long)%360
    return drik.dasavarga_from_long(_tri_sphuta, divisional_chart_factor=mixed_dvf)
    
def tri_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,
               chart_method=1,years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    asc_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
    gulika_long = gulika[0]*30+gulika[1]
    _tri_sphuta = (moon_long+asc_long+gulika_long)%360
    return drik.dasavarga_from_long(_tri_sphuta, divisional_chart_factor=divisional_chart_factor)
def chatur_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tri_sphuta= tri_sphuta_mixed_chart(dob, tob, place,varga_factor_1,chart_method_1,varga_factor_2,chart_method_2)
    _chatur_sphuta = (sun_long+_tri_sphuta[0]*30+_tri_sphuta[1])%360
    return drik.dasavarga_from_long(_chatur_sphuta, divisional_chart_factor=mixed_dvf)
def chatur_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,
               chart_method=1,years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tri_sphuta= tri_sphuta(dob, tob, place, ayanamsa_mode, divisional_chart_factor, chart_method,
                            years, months, sixty_hours)
    _chatur_sphuta = (sun_long+_tri_sphuta[0]*30+_tri_sphuta[1])%360
    return drik.dasavarga_from_long(_chatur_sphuta, divisional_chart_factor=divisional_chart_factor)
def pancha_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    rahu_long = planet_positions[8][1][0]*30+planet_positions[8][1][1]
    _chatur_sphuta= chatur_sphuta_mixed_chart(dob, tob, place,varga_factor_1,chart_method_1,varga_factor_2,chart_method_2)
    _pancha_sphuta = (rahu_long+_chatur_sphuta[0]*30+_chatur_sphuta[1])%360
    return drik.dasavarga_from_long(_pancha_sphuta, divisional_chart_factor=mixed_dvf)    
def pancha_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,
               chart_method=1,years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours)
    rahu_long = planet_positions[8][1][0]*30+planet_positions[8][1][1]
    _chatur_sphuta= chatur_sphuta(dob, tob, place, ayanamsa_mode, divisional_chart_factor, chart_method,
                                  years, months, sixty_hours)
    _pancha_sphuta = (rahu_long+_chatur_sphuta[0]*30+_chatur_sphuta[1])%360
    return drik.dasavarga_from_long(_pancha_sphuta, divisional_chart_factor=divisional_chart_factor)
def prana_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    asc_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=mixed_dvf)
    gulika_long = gulika[0]*30+gulika[1]
    _prana_long = (asc_long*5 + gulika_long) %360
    return drik.dasavarga_from_long(_prana_long, divisional_chart_factor=mixed_dvf)
def prana_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                 years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours)
    asc_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
    gulika_long = gulika[0]*30+gulika[1]
    _prana_long = (asc_long*5 + gulika_long) %360
    return drik.dasavarga_from_long(_prana_long, divisional_chart_factor=divisional_chart_factor)
def deha_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=mixed_dvf)
    gulika_long = gulika[0]*30+gulika[1]
    _deha_long = (moon_long*8 + gulika_long) %360
    return drik.dasavarga_from_long(_deha_long, divisional_chart_factor=mixed_dvf)
def deha_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor, chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
    gulika_long = gulika[0]*30+gulika[1]
    _deha_long = (moon_long*8 + gulika_long) %360
    return drik.dasavarga_from_long(_deha_long, divisional_chart_factor=divisional_chart_factor)
def mrityu_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=mixed_dvf)
    gulika_long = gulika[0]*30+gulika[1]
    _mrityu_long = (gulika_long*7 + sun_long) %360
    return drik.dasavarga_from_long(_mrityu_long, divisional_chart_factor=mixed_dvf)
def mrityu_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                  years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    gulika = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
    gulika_long = gulika[0]*30+gulika[1]
    _mrityu_long = (gulika_long*7 + sun_long) %360
    return drik.dasavarga_from_long(_mrityu_long, divisional_chart_factor=divisional_chart_factor)
def sookshma_tri_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    _prana_long = prana_sphuta_mixed_chart(dob, tob, place,varga_factor_1,chart_method_1,varga_factor_2,chart_method_2)
    _deha_long = deha_sphuta_mixed_chart(dob, tob, place,varga_factor_1,chart_method_1,varga_factor_2,chart_method_2)
    _mrityu_long = mrityu_sphuta_mixed_chart(dob, tob, place,varga_factor_1,chart_method_1,varga_factor_2,chart_method_2)
    _sookshma_long = (_prana_long[0]*30+_prana_long[1] + _deha_long[0]*30+_deha_long[1] + _mrityu_long[0]*30+_mrityu_long[1]) %360
    return drik.dasavarga_from_long(_sookshma_long, divisional_chart_factor=mixed_dvf)
def sookshma_tri_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,
                        chart_method=1,years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    _prana_long = prana_sphuta(dob, tob, place, ayanamsa_mode, divisional_chart_factor, chart_method,years, months, sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    _deha_long = deha_sphuta(dob, tob, place, ayanamsa_mode, divisional_chart_factor,chart_method, years, months, sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    _mrityu_long = mrityu_sphuta(dob, tob, place, ayanamsa_mode, divisional_chart_factor, chart_method,years, months, sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    _sookshma_long = (_prana_long[0]*30+_prana_long[1] + _deha_long[0]*30+_deha_long[1] + _mrityu_long[0]*30+_mrityu_long[1]) %360
    return drik.dasavarga_from_long(_sookshma_long, divisional_chart_factor=divisional_chart_factor)
def beeja_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    jupiter_long = planet_positions[5][1][0]*30+planet_positions[5][1][1]
    venus_long = planet_positions[6][1][0]*30+planet_positions[6][1][1]
    _beeja_long = (sun_long + jupiter_long + venus_long)%360
    return drik.dasavarga_from_long(_beeja_long, divisional_chart_factor=mixed_dvf)
def beeja_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                 years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    jupiter_long = planet_positions[5][1][0]*30+planet_positions[5][1][1]
    venus_long = planet_positions[6][1][0]*30+planet_positions[6][1][1]
    _beeja_long = (sun_long + jupiter_long + venus_long)%360
    return drik.dasavarga_from_long(_beeja_long, divisional_chart_factor=divisional_chart_factor)
def kshetra_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    jupiter_long = planet_positions[5][1][0]*30+planet_positions[5][1][1]
    mars_long = planet_positions[3][1][0]*30+planet_positions[3][1][1]
    _kshetra_long = (moon_long + jupiter_long + mars_long)%360
    return drik.dasavarga_from_long(_kshetra_long, divisional_chart_factor=mixed_dvf)
def kshetra_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                   years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    jupiter_long = planet_positions[5][1][0]*30+planet_positions[5][1][1]
    mars_long = planet_positions[3][1][0]*30+planet_positions[3][1][1]
    _kshetra_long = (moon_long + jupiter_long + mars_long)%360
    return drik.dasavarga_from_long(_kshetra_long, divisional_chart_factor=divisional_chart_factor)
def tithi_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tithi_long = (moon_long - sun_long) %360
    return drik.dasavarga_from_long(_tithi_long, divisional_chart_factor=mixed_dvf)
def tithi_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                 years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tithi_long = (moon_long - sun_long) %360
    return drik.dasavarga_from_long(_tithi_long, divisional_chart_factor=divisional_chart_factor)
def yoga_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1,
                            add_yogi_longitude=False):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    yogi_long = 93+20/60 if add_yogi_longitude else 0
    _yoga_long = (moon_long + sun_long + yogi_long) %360
    return drik.dasavarga_from_long(_yoga_long, divisional_chart_factor=mixed_dvf)
def yoga_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                years=1,months=1,sixty_hours=1,add_yogi_longitude=False,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    moon_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    yogi_long = 93+20/60 if add_yogi_longitude else 0
    _yoga_long = (moon_long + sun_long + yogi_long) %360
    return drik.dasavarga_from_long(_yoga_long, divisional_chart_factor=divisional_chart_factor)
def yogi_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    return yoga_sphuta_mixed_chart(dob, tob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2, 
                                   add_yogi_longitude=True)
def yogi_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    return yoga_sphuta(dob,tob,place,ayanamsa_mode,divisional_chart_factor,chart_method,
                       years,months,sixty_hours,add_yogi_longitude=True,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
def avayogi_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    mixed_dvf = varga_factor_1*varga_factor_2
    yl = yogi_sphuta_mixed_chart(dob, tob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    ayl = (yl[0]*30+yl[1]+186+40/60)%360
    return drik.dasavarga_from_long(ayl, mixed_dvf)
def avayogi_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                   years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    yl = yogi_sphuta(dob,tob,place,ayanamsa_mode,divisional_chart_factor,chart_method,years,months,sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    ayl = (yl[0]*30+yl[1]+186+40/60)%360
    return drik.dasavarga_from_long(ayl, divisional_chart_factor)
def rahu_tithi_sphuta_mixed_chart(dob,tob,place,varga_factor_1=1,chart_method_1=1,varga_factor_2=1,chart_method_2=1):
    jd_at_dob = utils.julian_day_number(dob, tob)
    mixed_dvf = varga_factor_1*varga_factor_2
    planet_positions = charts.mixed_chart(jd_at_dob, place, varga_factor_1, chart_method_1, varga_factor_2, chart_method_2)
    rahu_long = planet_positions[8][1][0]*30+planet_positions[8][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tithi_long = (rahu_long - sun_long) %360
    return drik.dasavarga_from_long(_tithi_long, divisional_chart_factor=mixed_dvf)
def rahu_tithi_sphuta(dob,tob,place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE,divisional_chart_factor=1,chart_method=1,
                      years=1,months=1,sixty_hours=1,base_rasi=None,count_from_end_of_sign=None):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=ayanamsa_mode, 
                                        divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                        years=years,months=months, sixty_hours=sixty_hours,
                                        base_rasi=base_rasi,count_from_end_of_sign=count_from_end_of_sign)
    rahu_long = planet_positions[8][1][0]*30+planet_positions[8][1][1]
    sun_long = planet_positions[1][1][0]*30+planet_positions[1][1][1]
    _tithi_long = (rahu_long - sun_long) %360
    return drik.dasavarga_from_long(_tithi_long, divisional_chart_factor=divisional_chart_factor)
    #return _tithi_long
if __name__ == "__main__":
    utils.set_language('en')
    dob = drik.Date(1996,12,7); tob = (10,34,0); place = drik.Place('Chennai,India',13.0878,80.2785,5.5)
    jd = utils.julian_day_number(dob, tob)
    print(tri_sphuta(dob, tob, place))
    exit()
    from jhora.tests import pvr_tests
    pvr_tests.sphuta_tests()
